let Remote = require('remote');
let App = Remote.require('app');
let Menu = Remote.require('menu');
let Dialog = Remote.require('dialog');
let MenuItem = Remote.require('menu-item');
let BrowserWindow = Remote.require('browser-window');

let Core = require('../modules/Core');

function AppMenus() {
  this._isKanomiCodeEnabled = false;
}

AppMenus.prototype = {
  rebuild: function() {
    this._isKanomiCodeEnabled = true;
    this.build();
  },

  build: function() {
    var templates = this._getMenusTemplateForCurrentPlatform();
    var menus = Menu.buildFromTemplate(templates);
    Menu.setApplicationMenu(menus);
  },

  _getMenuTemplateForAbout: function() {
    return {
      label: 'Vectr',
      submenu: [
        {
          label: 'About Vectr',
          selector: 'orderFrontStandardAboutPanel:',
        },
        {
          type: 'separator'
        },
        {
          label: 'Services',
          submenu: []
        },
        {
          type: 'separator'
        },
        {
          label: 'Hide Vectr',
          accelerator: 'CmdOrCtrl+H',
          selector: 'hide:'
        },
        {
          label: 'Hide Others',
          accelerator: 'CmdOrCtrl+Shift+H',
          selector: 'hideOtherApplications:'
        },
        {
          label: 'Show All',
          selector: 'unhideAllApplications:'
        },
        {
          type: 'separator'
        },
        {
          label: 'Quit',
          accelerator: 'CmdOrCtrl+Q',
          click: function() {
            App.quit();
          }
        }
      ]
    };
  },

  _getMenuTemplateForEdit: function() {
    return {
      label: 'Edit',
      submenu: [
        {
          label: 'Undo',
          accelerator: 'CmdOrCtrl+Z',
          selector: 'undo:'
        },
        {
          label: 'Redo',
          accelerator: 'Shift+CmdOrCtrl+Z',
          selector: 'redo:'
        },
        {
          type: 'separator'
        },
        {
          label: 'Cut',
          accelerator: 'CmdOrCtrl+X',
          selector: 'cut:'
        },
        {
          label: 'Copy',
          accelerator: 'CmdOrCtrl+C',
          selector: 'copy:'
        },
        {
          label: 'Paste',
          accelerator: 'CmdOrCtrl+V',
          selector: 'paste:'
        },
        {
          label: 'Select All',
          accelerator: 'CmdOrCtrl+A',
          selector: 'selectAll:'
        },
      ]
    };
  },

  _getMenuTemplateForView: function() {
    var submenu = [];

    if (this._isKanomiCodeEnabled || Core.isDev()) {
      submenu.push({
        label: 'Toggle Developer Tool for Main View',
        accelerator: 'Alt+CmdOrCtrl+I',
        click: function() {
          BrowserWindow.getFocusedWindow().toggleDevTools();
        }
      });

      submenu.push({
        label: 'Toggle Developer Tools for Web View',
        accelerator: 'Alt+CmdOrCtrl+W',
        click: function() {
          var webview = document.querySelector('webview');
          if (webview.isDevToolsOpened()) {
            webview.closeDevTools();
          } else {
            webview.openDevTools();
          }
        }
      });
    }

    submenu.push({
      label: 'Reload Application',
      accelerator: 'CmdOrCtrl+R',
      click: function() {
        BrowserWindow.getFocusedWindow().reloadIgnoringCache();
      }
    });

    submenu.push({
      label: 'Show Commit Id',
      click: function() {
        Dialog.showMessageBox({
          type: 'info',
          message: 'Current Commit Id - ' + Core.getCommitId(),
          buttons: ['Ok']
        });
      }
    });

    return {
      label: 'View',
      submenu: submenu
    };
  },

  _getMenuTemplateForWindow: function() {
    return {
      label: 'Window',
      role: 'window',
      submenu: [
        {
          label: 'Minimize',
          accelerator: 'CmdOrCtrl+M',
          selector: 'performMiniaturize:'
        },
        {
          label: 'Close',
          accelerator: 'CmdOrCtrl+W',
          click: function() {
            App.quit();
          }
        },
        {
          type: 'separator'
        },
        {
          label: 'Bring All to Front',
          selector: 'arrangeInFront:'
        }
      ]
    };
  },

  _getMenusTemplateForCurrentPlatform: function() {
    var templates = [];
    var platform = process.platform;

    switch (platform) {
      case 'darwin':
        templates.push(this._getMenuTemplateForAbout());
        templates.push(this._getMenuTemplateForEdit());
        templates.push(this._getMenuTemplateForView());
        templates.push(this._getMenuTemplateForWindow());
        break;

      default:
        templates.push(this._getMenuTemplateForEdit());
        templates.push(this._getMenuTemplateForView());
        break;
    }

    return templates;
  }
};

module.exports = new AppMenus();
